@@ -1,3 +1,4 @@
+//go:build nrf52840_mdk
 // +build nrf52840_mdk
 
 package machine
@@ -18,11 +19,6 @@ const (
 	UART_RX_PIN Pin = 19
 )
 
-// UART0 is the USB device
-var (
-	UART0 = USB
-)
-
 // I2C pins (unused)
 const (
 	SDA_PIN = NoPin
@@ -39,7 +35,7 @@ const (
 // USB CDC identifiers
 const (
 	usb_STRING_PRODUCT      = "Makerdiary nRF52840 MDK"
-	usb_STRING_MANUFACTURER = "Makerdiary"
+	usb_STRING_MANUFACTURER = "Nordic Semiconductor ASA"
 )
 
 var (
